package repositories

import (
	"app/models"

	"gorm.io/gorm"
)

type Carts_items interface {
	Create(*models.Cart_items) error
	GetItems(id uint, status string) ([]models.Cart_items, error)
	IncreaseQuantity(id uint) error
	DecreaseQuantity(id uint) error
	UpdateStatus(id uint) error
	Remove(id uint) error
}

type Cart_items_repository struct {
	DB *gorm.DB
}

func newCartsItemsRepository(db *gorm.DB) Carts_items {
	return &Cart_items_repository{DB: db}
}

func (r *Cart_items_repository) Create(cart_items *models.Cart_items) error {
	if err := r.DB.Create(&cart_items).Error; err != nil {
		return err
	}

	return nil
}

func (r *Cart_items_repository) GetItems(id uint, status string) ([]models.Cart_items, error) {
	var items []models.Cart_items
	if err := r.DB.Where("is_removed = ?", false).Where("cart_id = ? ", id).Where("status = ?", status).Find(&items).Error; err != nil {
		return nil, err
	}

	return items, nil
}

func (r *Cart_items_repository) IncreaseQuantity(id uint) error {
	var item models.Cart_items
	if err := r.DB.Where("is_removed = ?", false).Where("status = ? ", "pending").Where("cart_item_id = ?", id).First(&item).Error; err != nil {
		return err
	}

	quantity := item.Quantity + 1

	if err := r.DB.Where("is_removed = ? ", false).Where("cart_item_id = ?", id).Model(&item).Update("quantity", quantity).Error; err != nil {
		return err
	}

	if err := r.DB.Where("is_removed = ? ", false).Where("cart_item_id = ?", id).Model(&item).Update("price", item.PricePerUnit*uint64(quantity)).Error; err != nil {
		return err
	}

	return nil
}

func (r *Cart_items_repository) DecreaseQuantity(id uint) error {
	var item, temp models.Cart_items
	if err := r.DB.Where("is_removed = ?", false).Where("status = pending").Where("cart_item_id = ?", id).First(&item).Error; err != nil {
		return err
	}

	if item.Quantity > 1 {
		quantity := item.Quantity - 1
		price := item.Price * uint64(quantity)

		if err := r.DB.Where("is_removed = ? ", false).Where("cart_item_id = ?", id).Model(&temp).Update("quantity", quantity).Error; err != nil {
			return err
		}

		if err := r.DB.Where("is_removed = ? ", false).Where("cart_item_id = ?", id).Model(&temp).Update("price", price).Error; err != nil {
			return err
		}
	}

	return nil
}

func (r *Cart_items_repository) UpdateStatus(id uint) error {
	var item models.Cart_items
	if err := r.DB.Where("is_removed = ?", false).Where("cart_item_id = ?", id).Model(&item).Update("status", "purchased").Error; err != nil {
		return err
	}

	return nil
}

func (r *Cart_items_repository) Remove(id uint) error {
	var cart_items models.Cart_items
	if err := r.DB.Where("cart_item_id = ?", id).Model(&cart_items).Update("is_removed", true).Error; err != nil {
		return err
	}

	return nil
}
