package repositories

import (
	"app/models"

	"gorm.io/gorm"
)

type Phone interface {
	Create(p *models.Phone) error
	GetPhones(id uint) (*models.Phone, error)
	UpdatePhoneNumber(number string, id uint) error
	UpdateCellPhone(number string, id uint) error
	Delete(id uint) error
}

type Phone_repository struct {
	DB *gorm.DB
}

func newPhoneRepository(db *gorm.DB) Phone {
	return &Phone_repository{DB: db}
}

func (r *Phone_repository) Create(phone *models.Phone) error {
	if err := r.DB.Create(&phone).Error; err != nil {
		return err
	}

	return nil
}

func (r *Phone_repository) GetPhones(id uint) (*models.Phone, error) {
	var phone models.Phone
	if err := r.DB.Where("is_removed = ?", false).Where("user_id = ?", id).Find(&phone).Error; err != nil {
		return nil, err
	}

	return &phone, nil
}

func (r *Phone_repository) UpdatePhoneNumber(number string, id uint) error {
	var phone models.Phone
	if err := r.DB.Where("is_removed = ? ", false).Where("phone_id = ? ", id).Model(&phone).Update("phone_number", number).Error; err != nil {
		return err
	}

	return nil
}

func (r *Phone_repository) UpdateCellPhone(number string, id uint) error {
	var phone models.Phone
	if err := r.DB.Where("is_removed = ? ", false).Where("phone_id = ? ", id).Model(&phone).Update("cell_phone_number", number).Error; err != nil {
		return err
	}

	return nil
}

func (r *Phone_repository) Delete(id uint) error {
	var phone models.Phone
	if err := r.DB.Where("phone_id = ?", id).Model(&phone).Update("is_removed", true).Error; err != nil {
		return err
	}

	return nil
}
